/////////////////////////////////////////////////////////////////////////////////////////
//
// MC9S08QG4 sample for Freescale USBSPYDER08
//
// This program continuously varies the brightness of the LED by using a Pulse Width
// Modulation (PWM) technique.
//
// If you turn an LED on and off fast enough then it will appear to stay on continuously 
// and since there is less current flowing overall, the LED will appear less bright.
// By varying the duty cycle of the signal used to turn the LED on and off, we can vary
// the LED brightness.
// In this example, the LED varies its brightness from a minimum value to a maximum value
// and back to a minum value every second and a half. During this period, the LED is
// driven on and off about 200 times, each time with different on/off durations.
// --------------------------------------------------------------------------------------
//
// This project has been written for CodeWarrior 5.1 for HC(S)08
// and uses Registers Files MC9S08QG4.H and MC9S08QG4.C, version 2.87.095
//         												  
// --------------------------------------------------------------------------------------
// Copyright (c) 2006 SofTec Microsystems
// http://www.softecmicro.com/
//
/////////////////////////////////////////////////////////////////////////////////////////

#include <hidef.h>
#include "derivative.h"

#define TIMER_STOP                0x00
#define TIMER_START               0x4A  // Configures Timer1 with prescaler 4 and fBUS as source cock
    
#define DUTY_CYCLE_MAX            98    // %
#define DUTY_CYCLE_MIN            2     // %       
#define TIMER_OVERFLOW            15000 // us
unsigned char DUTY_CYCLE_INCREMENT=      1;     // %

// ICSTRM location in Flash
unsigned char ICSTRM_FLASH @0xFFAF;

// Flash security (NVOPT Register)
const unsigned char FLASH_SECURITY  @0xFFBF = 0xFE;

unsigned int duty_cycle_variation;
unsigned char alea_tabMM[64];
unsigned char alea_ptMM=56, alea_pt24MM=32, alea_pt55MM=1;
unsigned char alea,alea_avg2,alea_avg3,avg2[2],avg3[3];


void rnd_init(unsigned char seed)  {
          unsigned int tmp1;
          unsigned char i;
          alea_tabMM[0]=seed;
          
          for (i=1;i<64;i++)                   {
          tmp1=((unsigned int)137*alea_tabMM[i-1]+(unsigned int)187) ;
          tmp1=tmp1 % 256 ;
            
          alea_tabMM[i]= (unsigned char) tmp1;
                                               }
                                             } 
unsigned char rnd(void)
    {
    unsigned char x;
         unsigned int tmp;
         tmp = (unsigned int)(alea_tabMM[alea_pt24MM] +       // alea de 0  255; tmp de 0 512
                              alea_tabMM[alea_pt55MM])%255;       
         alea_tabMM[alea_ptMM] = (unsigned char)(tmp);
         x = alea_tabMM[alea_ptMM];
         
         alea_ptMM   = (alea_ptMM + 1) & 63;
         alea_pt24MM = (alea_pt24MM + 1) & 63;
         alea_pt55MM = (alea_pt55MM + 1) & 63;

         return x;
    }

 


 
  
/////////////////////////////////////////////////////////////////////////////////////////
// PeriphInit
// --------------------------------------------------------------------------------------
// Initializes various registers and peripherals
/////////////////////////////////////////////////////////////////////////////////////////
void PeriphInit (void)
{  
  // Disables COP
  SOPT1 &= 0x7F;
  
  // Configures PTA0 as output LED
  PTAD = 0x00;
  PTADD = 0x01;
  
  // Configures ICS clock. Selects FLL Engaged Internal Mode (Divides ICSOUT by 2)
  // fBUS about 4 MHz
  ICSTRM = ICSTRM_FLASH;
  ICSC2 = 0x40;
  ICSC1 = 0x04;
  // Waits until output FLL is selected (CLKST=00)
  while (ICSSC & 0x0C)
    ; 
       
  // Timer1 period
  TPMMOD = TIMER_OVERFLOW;
  // Initial duty cycle 
  TPMC0V = TIMER_OVERFLOW / 100 * DUTY_CYCLE_MIN;  
  // Selects PWM Low-true pulses in the Timer1 Channel0 (PTA0/TPM1CH0) 
  TPMC0SC = 0x024;
  // Starts Timer1 
  TPMSC = TIMER_START;
}

/////////////////////////////////////////////////////////////////////////////////////////
// MAIN
// --------------------------------------------------------------------------------------
// Entry point
/////////////////////////////////////////////////////////////////////////////////////////
void main (void) 
{
  int pause ;
  
  EnableInterrupts;
  PeriphInit();


  
  
  rnd_init(127);

  for(;;) {
  
   for (pause=1; pause<15000; pause++) ;
  
  alea=rnd();
  
  avg2[0]=avg2[1]            ;
  avg2[1]=alea    /2          ;
  alea_avg2=   avg2[0] + avg2[1] ;
  
  avg3[0]=avg3[1]             ;
  avg3[1]=avg3[2]               ;
  avg3[2]=alea    /3              ;
  
  
  alea_avg3=   avg3[0] + avg3[1] + avg3[2]   ;
  
  
        }
  
  
}

/////////////////////////////////////////////////////////////////////////////////////////
// Timer1Overflow
// --------------------------------------------------------------------------------------
// TIMER1 Overflow Interrupt 
/////////////////////////////////////////////////////////////////////////////////////////
interrupt  VectorNumber_Vtpmovf void Timer1Overflow (void)
{
  // Stops Timer1
  TPMSC = TIMER_STOP;
  
  // Updates
  TPMC0V = TIMER_OVERFLOW / 255 * (alea_avg3/2+alea_avg2/2) ;

  // Restarts Timer1
  TPMSC = TIMER_START;  
}